/**
 * \par Copyright (C), 2012-2016, MakeBlock
 * @file    Me_Megapi_encoder_pid_speed.ino
 * @author  MakeBlock
 * @version V1.0.0
 * @date    2016/07/14
 * @brief   Description: this file is sample code for Megapi encoder motor device.
 *
 * Function List:
 *    1. uint8_t MeEncoderOnBoard::getPortB(void);
 *    2. uint8_t MeEncoderOnBoard::getIntNum(void);
 *    3. void MeEncoderOnBoard::pulsePosPlus(void);
 *    4. void MeEncoderOnBoard::pulsePosMinus(void);
 *    5. void MeEncoderOnBoard::setMotorPwm(int pwm);
 *    6. double MeEncoderOnBoard::getCurrentSpeed(void);
 *    7. void MeEncoderOnBoard::setSpeedPid(float p,float i,float d);
 *    8. void MeEncoderOnBoard::setPosPid(float p,float i,float d);
 *    7. void MeEncoderOnBoard::setPosPid(float p,float i,float d);
 *    8. void MeEncoderOnBoard::setPulse(int16_t pulseValue);
 *    9. void MeEncoderOnBoard::setRatio(int16_t RatioValue);
 *    10. void MeEncoderOnBoard::runSpeed(float speed);
 *    11. void MeEncoderOnBoard::loop(void);
 *
 * \par History:
 * <pre>
 * <Author>     <Time>        <Version>      <Descr>
 * Mark Yan     2016/07/14    1.0.0          build the new
 * </pre>
 */

#include <MeMegaPi.h>
#include <SoftwareSerial.h>
#include <Wire.h>

/*Declare Board ID*/
MeEncoderOnBoard Encoder_1(SLOT1); /* Rechter Motor */
MeEncoderOnBoard Encoder_2(SLOT2); /* Linker Motor */
MeEncoderOnBoard Encoder_3(SLOT3); /* Arm Motor */
MeMegaPiDCMotor handMotor(PORT4B);
MeInfraredReceiver infraredReceiverDecode(PORT_6);
MeWifi Wifi(PORT_8);

/* Definition of Variables*/
int moveSpeed = 200;
int stopSpeed = 0;
int armSpeed = 50;
int gripperSpeed = 50;

void isr_process_encoder1(void)
{
  if(digitalRead(Encoder_1.getPortB()) == 0)
  {
    Encoder_1.pulsePosMinus();
  }
  else
  {
    Encoder_1.pulsePosPlus();;
  }
}

void isr_process_encoder2(void)
{
  if(digitalRead(Encoder_2.getPortB()) == 0)
  {
    Encoder_2.pulsePosMinus();
  }
  else
  {
    Encoder_2.pulsePosPlus();
  }
}
void isr_process_encoder3(void)
{
  if(digitalRead(Encoder_3.getPortB()) == 0)
  {
    Encoder_3.pulsePosMinus();
  }
  else
  {
    Encoder_3.pulsePosPlus();
  }
}

void setup()
{
  infraredReceiverDecode.begin();
  attachInterrupt(Encoder_1.getIntNum(), isr_process_encoder1, RISING);
  attachInterrupt(Encoder_2.getIntNum(), isr_process_encoder2, RISING);
  attachInterrupt(Encoder_3.getIntNum(), isr_process_encoder3, RISING);
  Serial.begin(115200);
  Wifi.begin(115200);
  
  //Set PWM 8KHz
  TCCR1A = _BV(WGM10);
  TCCR1B = _BV(CS11) | _BV(WGM12);

  TCCR2A = _BV(WGM21) | _BV(WGM20);
  TCCR2B = _BV(CS21);

  Encoder_1.setPulse(7);
  Encoder_2.setPulse(7);
  Encoder_3.setPulse(7);
  Encoder_1.setRatio(26.9);
  Encoder_2.setRatio(26.9);
  Encoder_3.setRatio(26.9);
  Encoder_1.setPosPid(1.8,0,1.2);
  Encoder_2.setPosPid(1.8,0,1.2);
  Encoder_3.setPosPid(1.8,9,1.2);
  Encoder_1.setSpeedPid(0.18,0,0);
  Encoder_2.setSpeedPid(0.18,0,0);
  Encoder_3.setSpeedPid(0.18,0,0);
}

void loop()
{
 uint8_t ReceiverCode; 
  if(infraredReceiverDecode.available()||infraredReceiverDecode.buttonState())
  {
    ReceiverCode = infraredReceiverDecode.read();
    switch(ReceiverCode)
    {
      case IR_BUTTON_SETTING:
      Encoder_1.runSpeed(stopSpeed);
      Encoder_2.runSpeed(stopSpeed);
      break;
      case IR_BUTTON_UP:
      Encoder_1.runSpeed(moveSpeed);
      Encoder_2.runSpeed(-moveSpeed);
      break;
      case IR_BUTTON_DOWN:
      Encoder_1.runSpeed(-moveSpeed);
      Encoder_2.runSpeed(moveSpeed);
      break;
      case IR_BUTTON_RIGHT:
      Encoder_1.runSpeed(-moveSpeed);
      Encoder_2.runSpeed(-moveSpeed);
      break;
      case IR_BUTTON_LEFT:
      Encoder_1.runSpeed(moveSpeed);
      Encoder_2.runSpeed(moveSpeed);
      break;
      case IR_BUTTON_4:
      handMotor.run(gripperSpeed); /*Include while Loop and setup function call to define max. time motor should move for opening and closing give speed */
      break;
      case IR_BUTTON_6:
      handMotor.run(-gripperSpeed);
      break;
      case IR_BUTTON_5:
      handMotor.run(stopSpeed); /*Max_bzw. Min_Value Definieren und automatisches Abschalten*/
      break;
      case IR_BUTTON_1:
      Encoder_3.runSpeed(armSpeed); /* Include While Loop and setup function call to define min max position before collision of arm with chassis*/
      break;
      case IR_BUTTON_3:
      Encoder_3.runSpeed(-armSpeed);
      break;
      case IR_BUTTON_2:
      Encoder_3.runSpeed(stopSpeed);
      break;
      default:
      break;
    }
  
  }
  Encoder_1.loop();
  Encoder_2.loop();
  Encoder_3.loop();

  if (Serial.available())
  {
    char outDat;
    outDat=Serial.read();
    Wifi.write(outDat);
  }
  
  /*
  Serial.print("Spped 1:");
  Serial.print(Encoder_1.getCurrentSpeed());
  Serial.print(" ,Spped 2:");
  Serial.println(Encoder_2.getCurrentSpeed());
  Serial.print("  ,Spped 3:");
  Serial.println(Encoder_3.getPulsePos());*/
}
